@@ -1,5 +1,6 @@
 from __future__ import division, print_function, absolute_import
 from . import aggregator
+from . import affinitymat
 from . import coordproducers
 from . import core
 from . import gammamix
@@ -11,3 +12,4 @@ from . import value_provider
 from . import clusterinit
 from . import seqlet_embedding
 from . import cluster
+from . import pattern_filterer
